@@ -7,10 +7,13 @@ import fcntl
 import os
 import shlex
 import sys
+from pathlib import Path
 from typing import Any, List, Optional, Sequence
 
 from blight.exceptions import BlightError
 
+SWIZZLE_SENTINEL = "@blight-swizzle@"
+
 
 def die(message):
     """
@@ -113,6 +116,16 @@ def flock_append(filename):
             fcntl.flock(io, fcntl.LOCK_UN)
 
 
+def unswizzled_path() -> str:
+    """
+    Returns a version of the current `$PATH` with any blight shim paths removed.
+    """
+    paths = os.getenv("PATH", "").split(os.pathsep)
+    paths = [p for p in paths if not Path(p).name.endswith(SWIZZLE_SENTINEL)]
+
+    return os.pathsep.join(paths)
+
+
 def load_actions():
     """
     Loads any blight actions requested via the environment.
@@ -160,7 +173,7 @@ def load_actions():
         action_config = os.getenv(f"BLIGHT_ACTION_{action_name.upper()}", None)
         if action_config is not None:
             action_config = shlex.split(action_config)
-            action_config = dict(c.split("=") for c in action_config)
+            action_config = dict(c.split("=", 1) for c in action_config)
         else:
             action_config = {}
 
